import webapp2
import os
import jinja2
import logging
import datetime
from datetime import date
from google.appengine.ext import db

# Initiate jinja
template_directory = os.path.join(os.path.dirname(__file__), 'templates')
jinja_environment = jinja2.Environment(loader = jinja2.FileSystemLoader(template_directory),
                                       autoescape = True)

# Entry entity
class Entry(db.Model):
    hg_url = db.StringProperty()
    status = db.StringProperty()
    last_checked = db.DateTimeProperty(auto_now_add = True)
    
    @classmethod
    def create(cls, hg_url, status):
        return cls(hg_url = hg_url, status = status)
    
    @classmethod
    def get_last_entry(cls, hg_url):
        return cls.all().filter('hg_url = ', hg_url).order('-last_checked').get()

# Helper handler
class Handler(webapp2.RequestHandler):
    def write(self, *a, **kw):
        self.response.out.write(*a, **kw)
        
    def render_str(self, template, **params):
        jinja_environment.filters['datetimeformat'] = datetimeformat
        t = jinja_environment.get_template(template)
        return t.render(params)
    
    def render(self, template, **kw):
        self.write(self.render_str(template, **kw))

# Remove http or https from url
def convert_url(url):
    if url[:7] == "http://":
        url = url[7:]
    elif url[:8] == "https://":
        url = url[8:]
    return url
    
def datetimeformat(value, format="%b %d, %Y"):
    return value.strftime(format)
    
# Return all rows for a url with an entry date later than 1 month ago
def get_ups(url, last_month_date):
    return get_checks(url, last_month_date).filter("status = ", "up")

# Return all checks with a check date later than 1 month ago
def get_checks(url, last_month_date):
    return Entry.all().filter('hg_url = ', url).filter('last_checked > ', last_month_date)

# Return the percent uptime
def last_month_uptime(url, last_month_date):
    ups = get_ups(url, last_month_date).count()
    checks = get_checks(url, last_month_date).count()
    uptime = 100 * ups / float(checks)
    return "%.1f" % uptime

class SubmitInfoHandler(Handler):
    def get(self):
        try:
            hg_url = self.request.get("hgurl")
            status = self.request.get("status")
            if hg_url and status:
                url = convert_url(str(hg_url))
                new_entry = Entry.create(url, status)
                new_entry.put()
                logging.info(str(new_entry.last_checked) + " " + new_entry.status)
                self.write("Submission successful.")
            else:
                self.write("One or more of your parameters are missing (make sure to use hgurl and status).")
            
        except:
            self.write("There was a problem with your request.")
        
class GetInfoHandler(Handler):
    def get(self):
        try:
            hg_url = self.request.get("hgurl")
            if hg_url:
                url = convert_url(hg_url)
                today = date.today()
                last_month_date = datetime.date(today.year, (today.month - 1), today.day)
                num_checks = get_checks(url, last_month_date).count()    # See if there are checks in the last month
                
                if num_checks:
                    uptime = last_month_uptime(url, last_month_date)
                    params = dict(last_entry = Entry.get_last_entry(url), uptime = uptime)
                    self.render("display_text.html", **params)
                else:
                    self.write("There are no checks in the last month.")
            else:
                self.write("One or more of your parameters are missing (make sure to use hgurl).")
                
        except:
            self.write("There was a problem with your request.")

app = webapp2.WSGIApplication([
    ('/get', GetInfoHandler),
    ('/reg', SubmitInfoHandler)
], debug=True)
